/*
 * infoproviders.js
 *
 *  Created on: 08.02.2016
 *      Author: Holger Dahm
 *
 * the module provides all infoprovider embracing stuff.
 *
 * Note: add all infoproviders necessary to service your app
 */

// initialize available infoproviders
var weatherinfoprovider = require("./weatherinfoprovider");
var newsinfoprovider    = require("./newsinfoprovider");
var stockinfoprovider   = require("./stockinfoprovider");
var holidayinfoprovider = require("./holidayinfoprovider");

// used for any standard initialization
var infoproviders = [ weatherinfoprovider,
					  newsinfoprovider,
					  stockinfoprovider,
					  holidayinfoprovider ];



/*
 * standard initialization of infoproviders
 */

// (default) options used in this infoprovider
// as well as all info providers
// Note: call setOptions() before doing anything!
var options;

function setOptions(values) {
	options = values;

	infoproviders.forEach(function(infoProvider, index, array) {
		infoProvider.setOptions(options);
	});
}

// app controller used in this module
// as well as all info providers for
// json or xml request, caching as well as message to pebble send operations
// Note: call setAppController() before doing anything!
var appcontroller;

function setAppController(controller) {
	appcontroller = controller;

	infoproviders.forEach(function(infoProvider, index, array) {
		infoProvider.setAppController(appcontroller);
	});
}



/*
 * push still cached infos to appmessage queue
 * means, test if "old" information is available; in case send it to pebble
 */

function pushCachedInfosToAppmessageQueue() {
	appcontroller.pushLocalStorageToAppMessages(weatherinfoprovider.WEATHER_ID);
	appcontroller.pushLocalStorageToAppMessages(weatherinfoprovider.FORECAST_ID);

    if (appcontroller.isRequested(( options.financialMode ? "currencies" : "stock" ),
        options.showFinancials,
        ( options.financialMode ? options.currencies : options.stocks ),
        options.startFinancials, options.stopFinancials, false)) {
		appcontroller.pushLocalStorageToAppMessages(stockinfoprovider.STOCKS_ID);
	}
}

/*
 * clear all cached infos
 */

function clearCache() {
    localStorage.removeItem(weatherinfoprovider.WEATHER_ID);
    localStorage.removeItem(weatherinfoprovider.FORECAST_ID);
    localStorage.removeItem(stockinfoprovider.STOCKS_ID);
}

/*
 * central app processing in reaction to pebble request
 */

function process(selectionID) {
    try {
        // start processing; flag is unset in sendAppMessage if last element of queue is send
        appcontroller.setProcessing(true);

        switch (selectionID) {
            case 0:
            case 4:
            	weatherinfoprovider.fetch(0);
                break;
            case 1:
            	newsinfoprovider.fetch(2);
                break;
            case 2:
            	stockinfoprovider.fetch(0);
                break;
            case 3:
            	newsinfoprovider.fetch(1);
                break;
            case 5:
            	newsinfoprovider.fetch(0);
                break;
            case 6:
            	holidayinfoprovider.fetch(0);
                break;
            default:
                console.log("Error - received selection " + selectionID + " is not defined.");
            	appcontroller.setProcessing(false);
                break;
        }
    }
    catch (error) {
        console.log("Error during processing!\n" + error.stack);
        appcontroller.appMessageQueue.push({
            "message" : {
                "nodata" : "processing error"
            }
        });
        appcontroller.sendAppMessage(true); // unsets processing = true ;)
    }
}

function processPebbleRequest(event) {
    // Debug content - begin
	/*newsinfoprovider.testAllFeeds();
	return;*/
    // Debug content - end

	// test if processing is still ongoing or request is really requested
    var selectionID = event.payload.selectionID;
    console.log("Got data request! [selectionID: " + selectionID +  "]");

    if (appcontroller.isProcessing()) {
        // nothing remains to do
        console.log("Stopped because I'm still processing!");
        return;
    }

    if ( selectionID != 0 /*weather*/ &&
         selectionID != 4 /*weather forecast*/ ) {
    	var noDataString = "";

       if (options.showInfoPanels == 0) {
    	   noDataString = "weather mode";
       }
       else if (options.showInfoPanels == 2 && !appcontroller.pebblehelpers.isWeakday()) {
    	   noDataString = "weekend mode";
       }
       else if (!appcontroller.pebblehelpers.nowIsBetween(options.startInfoPanels, options.stopInfoPanels)) {
    	   noDataString = "no info time";
       }

       if (noDataString.length > 0) {
           appcontroller.appMessageQueue.push({
               "message" : {
                   "nodata" : noDataString
               }
           });
           appcontroller.sendAppMessage(true);

           console.log("Stopped - " + noDataString + "!");

           return;
       }
    }

    process(selectionID);
}



/*
 * transparent infoprovider interface
 */

module.exports = {
		// set (default) options used in this module
		// as well as all info providers
		setOptions : setOptions,

		// set app controller used in this module
		// as well as all info providers
		setAppController : setAppController,

		// push still cached infos to appmessage queue
		pushCachedInfosToAppmessageQueue : pushCachedInfosToAppmessageQueue,

		// clear all cached infos
		clearCache : clearCache,

		// process pebble request
		processPebbleRequest : processPebbleRequest
};
